from typing import Any, Dict, Optional

from xpanse.const import V3_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class TagsEndpoint(ExEndpoint):
    """
    Part of the Annotations v3 API for tagging assets.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of tag annotations. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
            pageToken (str, optional):
                Returns results starting at this Page Token.
            id (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            name (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.
            disabled (bool, optional):
                Boolean representing if the tag is currently disabled.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the tag results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Prints all tag objects:
            >>> for res in client.annotations.tags.v3.list():
            ...     for tag in res:
            ...         print(tag)
        """
        return ExResultIterator(self._api, f"{V3_PREFIX}/annotations/tags", kwargs)

    def get(self, tag_id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given tag ID. Arguments should be passed as keyword args using
        the names below.

        Args:
            tagId (str):
                The tag ID. This will be a uuid.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the tag.

        Examples:
            >>> # Returns a Tag.
            >>> domain = client.annotations.tags.v3.get(<tag_id>)
        """
        return self._api.get(
            f"{V3_PREFIX}/annotations/tags/{tag_id}", params=kwargs
        ).json()

    def create(
        self, name: str, description: Optional[str] = None, disabled: bool = False
    ) -> Dict[str, Any]:
        """
        Create a new tag.

        Args:
            name (str):
                Name of the tag. Limit of 128 characters.
            description (str, optional):
                Tag description. Limit of 512 characters.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the newly created tag.

        Examples:
            >>> # Create a new tag
            >>> tag = client.annotations.tags.v3.create(name="DMZ")
        """
        payload = {"name": name, "description": description, "disabled": disabled}
        return self._api.post(f"{V3_PREFIX}/annotations/tags", json=payload).json()

    def update(self, tag_id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Updates a tag, see keyword args for fields that can be updated.

        Note: You cannot delete tags, only disable them.

        Args:
            tag_id (str):
                UUID of the tag to be updated
            description (str, optional):
                Tag description. Limit of 512 characters.
            disabled (bool, optional):
                Whether the tag should be enabled or disabled.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the updated tag.

        Examples:
            >>> # Update a tag's disabled status
            >>> tag = client.annotations.tags.v3.update(disabled=True)
        """
        return self._api.put(
            f"{V3_PREFIX}/annotations/tags/{tag_id}", json={**kwargs}
        ).json()



class ExEndpoint:
    """
    The ExEndpoint class is used as a base class for all endpoints.
    Any additional logic that is desired to be present on all endpoints, but is
    outside of the scope of the session or client can be added here.
    """

    def __init__(self, session):
        self._api = session



from typing import Any, Dict, List, Union

from xpanse.const import V2_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.error import UnexpectedValueError
from xpanse.iterator import ExResultIterator


class DomainsEndpoint(ExEndpoint):
    """
    Part of the Assets v2 API for handling asset domains.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of asset domains. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
            pageToken (str, optional):
                Returns results starting at this Page Token.
            domainSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            formattedRegistrarName (str, optional):
                Comma-separated string; Returns only results that were found on the given formatted registrar names.
                If not set, filter is ignored.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            dnsResolutionStatus (str, optional):
                Comma-separated string; Returns only result whose asset's resolution statuses fall in the provided list.
                Valid values are `HAS_DNS_RESOLUTION`, `NO_DNS_RESOLUTION`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, `NO_ACTIVE_ON_PREM_SERVICE`.
            issueStatus (str, optional):
                Comma-separated string; Returns only result whose asset's issue statuses fall in the provided list.
                Valid values are `New`, `Investigating`, `In Progress`, `No Risk`, `Acceptable Risk`, `Resolved`
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (str, optional):
                Filter by whether the asset has a related cloud resource asset
            sort (str, optional):
                Comma-separated string; orders results by the given fields. If the field name is
                prefixed by a -, then the ordering will be descending for that field.
                Allowed values are `domain`, `-domain`, `dateAdded`, `-dateAdded`, `lastObserved`, `-lastObserved`.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.
            isPaidLevelDomain (bool, optional):
                `True` will return only top-level or paid level domains (i.e. example.com), `False` will return any
                subdomains(i.e. api.example.com). If omitted all domains will be returned.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the domain results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Prints all domain objects:
            >>> for res in client.assets.domains.v2.list():
            ...     for domain in res:
            ...         print(domain)
        """
        return ExResultIterator(self._api, f"{V2_PREFIX}/assets/domains", kwargs)

    def count(self, **kwargs: Any) -> int:
        """
        Returns the total count of Domains. This will return -1 if for some reason the count attribute
        is not returned in an otherwise valid response payload.

        Args:
            domainSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            formattedRegistrarName (str, optional):
                Comma-separated string; Returns only results that were found on the given formatted registrar names.
                If not set, filter is ignored.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            dnsResolutionStatus (str, optional):
                Comma-separated string; Returns only result whose asset's resolution statuses fall in the provided list.
                Valid values are `HAS_DNS_RESOLUTION`, `NO_DNS_RESOLUTION`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, `NO_ACTIVE_ON_PREM_SERVICE`.
            issueStatus (str, optional):
                Comma-separated string; Returns only result whose asset's issue statuses fall in the provided list.
                Valid values are `New`, `Investigating`, `In Progress`, `No Risk`, `Acceptable Risk`, `Resolved`.
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (str, optional):
                Filter by whether the asset has a related cloud resource asset.
            sort (str, optional):
                Comma-separated string; orders results by the given fields. If the field name is
                prefixed by a -, then the ordering will be descending for that field.
                Allowed values are `domain`, `-domain`, `dateAdded`, `-dateAdded`, `lastObserved`, `-lastObserved`.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.
            isPaidLevelDomain (bool, optional):
                `True` will return only top-level or paid level domains (i.e. example.com), `False` will return any
                subdomains(i.e. api.example.com). If omitted all domains will be returned.

        Returns:
            :int:
                The total count of domains.

        Examples:
            >>> # Print total count of domains containing `dev` substring.
            >>> print(client.assets.domains.v2.count(domainSearch="dev"))
        """
        return (
            self._api.get(f"{V2_PREFIX}/assets/domains/count", params=kwargs)
            .json()
            .get("count", -1)
        )

    def get(self, name: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given Domain. Arguments should be passed as keyword args using
        the names below.

        Args:
            name (str):
                Fully qualified domain name.
            minRecentIpLastObservedDate (str, optional):
                Filter by last observed timestamp for recent IPs. Date formatted string (YYYY-MM-DD).

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the domain.

        Examples:
            >>> # Return Domain.
            >>> domain = client.assets.domains.v2.get(<name>)
        """
        return self._api.get(f"{V2_PREFIX}/assets/domains/{name}", params=kwargs).json()

    def csv(self, file: str, **kwargs: Any) -> bool:
        """
        Downloads filtered domains as a csv file. Arguments should be passed as keyword args using
        the names below.

        Args:
            file (str):
                A relative path for saving the downloaded csv file.
            limit (int, optional):
                Returns at most this many results.
            domainSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            formattedRegistrarName (str, optional):
                Comma-separated string; Returns only results that were found on the given formatted registrar names.
                If not set, filter is ignored.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            dnsResolutionStatus (str, optional):
                Comma-separated string; Returns only result whose asset's resolution statuses fall in the provided list.
                Valid values are `HAS_DNS_RESOLUTION`, `NO_DNS_RESOLUTION`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, `NO_ACTIVE_ON_PREM_SERVICE`.
            issueStatus (str, optional):
                Comma-separated string; Returns only result whose asset's issue statuses fall in the provided list.
                Valid values are `New`, `Investigating`, `In Progress`, `No Risk`, `Acceptable Risk`, `Resolved`.
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (str, optional):
                Filter by whether the asset has a related cloud resource asset.
            isPaidLevelDomain (bool, optional):
                `True` will return only top-level or paid level domains (i.e. example.com), `False` will return any
                subdomains(i.e. api.example.com). If omitted all domains will be returned.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.

        Returns:
            :obj:`boolean`:
                `True` if the download was successful, otherwise `False`.

        Examples:
            >>> # Download a csv named `api-domains.csv` for all domains that contain `api`:
            >>> client.assets.domains.v2.csv(file="api-domains.csv", domainSearch="api")
        """
        return self._api.csv(
            path=f"{V2_PREFIX}/assets/domains/csv", file_=file, **kwargs
        )

    def bulk_tag(
        self,
        operation: str,
        asset_ids: List[str],
        tag_ids: List[str],
        return_raw: bool = False,
    ) -> Union[bool, Dict[str, Any]]:
        """
        Assigns or unassigns tags to assets.

        Args:
            operation (str):
                Operation type. Must be ASSIGN or UNASSIGN.
            asset_ids (list):
                A list of asset uuids to assign or unassign tags from.
            tag_ids (list):
                A list of tag uuids to assign or unassign to assets.
            return_raw (bool, optional):
                If False this will return a boolean response that reflects whether
                all of the operations were successful. If True the raw json response
                will be returned. Defaults to False.

        Returns:
            :bool:
                Returns a bool reflecting the operations success unless `return_raw`
                is true, in which case a dict is returned.

        Examples:
            >>> # Assign a tag to a domain
            >>> cli.assets.domains.bulk_tag("ASSIGN",
            ...                            ["8e589910-c1af-3c32-ae88-9a4b2dbcfe76"],
            ...                            ["f6164347-86d1-30cc-baf2-28bbb395403d"])
        """
        if operation not in ["ASSIGN", "UNASSIGN"]:
            raise UnexpectedValueError(
                f"The operation type '{operation}' is not valid.'"
            )
        payload: Dict[str, Any] = {"operations": []}
        for asset_id in asset_ids:
            payload["operations"].append(
                {"operationType": operation, "assetId": asset_id, "tagIds": tag_ids}
            )
        resp = self._api.post(
            f"{V2_PREFIX}/assets/domains/tag-assignments/bulk", json=payload
        ).json()
        if not return_raw:
            return resp.get("meta", {}).get("failureCount") == 0
        return resp

    def bulk_poc(
        self,
        operation: str,
        asset_ids: List[str],
        contact_ids: List[str],
        return_raw: bool = False,
    ) -> Union[bool, Dict[str, Any]]:
        """
        Assigns or unassigns Point-of-Contacts to assets.

        Args:
            operation (str):
                Operation type. Must be ASSIGN or UNASSIGN.
            asset_ids (list):
                A list of asset uuids to assign or unassign pocs from.
            contact_ids (list):
                A list of poc uuids to assign or unassign to assets.
            return_raw (bool, optional):
                If False this will return a boolean response that reflects whether
                all of the operations were successful. If True the raw json response
                will be returned. Defaults to False.

        Returns:
            :bool:
                Returns a bool reflecting the operations success unless `return_raw`
                is true, in which case a dict is returned.

        Examples:
            >>> # Assign a poc to a domain
            >>> cli.assets.domains.bulk_poc("ASSIGN",
            ...                            ["8e589910-c1af-3c32-ae88-9a4b2dbcfe76"],
            ...                            ["f6164347-86d1-30cc-baf2-28bbb395403d"])
        """
        if operation not in ["ASSIGN", "UNASSIGN"]:
            raise UnexpectedValueError(
                f"The operation type '{operation}' is not valid.'"
            )
        payload: Dict[str, Any] = {"operations": []}
        for asset_id in asset_ids:
            payload["operations"].append(
                {
                    "operationType": operation,
                    "assetId": asset_id,
                    "contactIds": contact_ids,
                }
            )
        resp = self._api.post(
            f"{V2_PREFIX}/assets/domains/contact-assignments/bulk", json=payload
        ).json()
        if not return_raw:
            return resp.get("meta", {}).get("failureCount") == 0
        return resp

    def annotation_update(
        self,
        domain_id: str,
        contacts: List[str] = [],
        tags: List[str] = [],
        note: str = "",
    ) -> Dict[str, Any]:
        """
        Updates the annotations for a single domain.

        Note: This will overwrite the existing annotations for an asset.
              If any arguments are omitted, that annotation type will be
              cleared for the asset.

        Args:
            domain_id (str):
                The uuid of the domain.
            contacts (list, optional):
                A list of poc emails to apply to the domain.
            tags (list, optional):
                A list of tag names to apply to the domain.
            note (str, optional):
                An optional note that can be added to the domain.

        Returns:
            :obj:`dict`:
                A dictionary containing the current annotations for the domain.

        Examples:
            >>> # Update annotations for a domain
            >>> cli.assets.domains.annotation_update(
            ...     domain_id="e5bdc732-522a-3864-8ff3-307d35f0f0a0",
            ...     contacts=["QA@testing.com"],
            ...     tags=["sdk_test"],
            ...     note="SDK Note")
        """
        payload: Dict[str, Any] = {"tags": [], "note": note or "", "contacts": []}
        for contact in contacts:
            payload["contacts"].append({"email": contact})
        for tag in [t.lower() for t in tags]:
            payload["tags"].append({"name": tag})
        return self._api.put(
            f"{V2_PREFIX}/assets/domains/{domain_id}/annotations", json=payload
        ).json()



from xpanse.base_api import ExApi
from xpanse.api.entities.v1.entities import EntitiesEndpoint


class EntitiesInnerApi(ExApi, EntitiesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return EntitiesEndpoint(self._api)


class EntitiesApi(ExApi):
    @property
    def entities(self):
        return EntitiesInnerApi(self._api)



from xpanse.base_api import ExApi
from xpanse.api.connectors.v1.services import ConnectorsServicesEndpoint
from xpanse.api.connectors.v1.accounts import ConnectorsAccountsEndpoint


class ConnectorsServicesAPI(ExApi, ConnectorsServicesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return ConnectorsServicesEndpoint(self._api)


class ConnectorsAccountsAPI(ExApi, ConnectorsAccountsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return ConnectorsAccountsEndpoint(self._api)


class ConnectorsApi(ExApi):
    @property
    def services(self):
        return ConnectorsServicesAPI(self._api)

    @property
    def accounts(self):
        return ConnectorsAccountsAPI(self._api)



from typing import Any, Dict, List, Union

from xpanse.const import V2_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.error import UnexpectedValueError
from xpanse.iterator import ExResultIterator


class CertificatesEndpoint(ExEndpoint):
    """
    Part of the Assets v2 API for handling asset certificates.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of asset certificates. Arguments should be passed as keyword args using
        the names below.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
            pageToken (str, optional):
                Returns results starting at this page Token.
            commonNameSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            property (str, optional):
                Comma-separated string; Returns only results whose certificate property falls in the provided list.
                NOTE: If omitted, API will return results for all properties the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            certificateAdvertisementStatus (str, optional):
                Comma-separated string; Returns only result whose asset's certificate advertisement statuses fall in the provided list.
                Valid values are `HAS_CERTIFICATE_ADVERTISEMENT` and `NO_CERTIFICATE_ADVERTISEMENT`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, and `NO_ACTIVE_ON_PREM_SERVICE`.
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (boolean, optional):
                Filter by whether the asset has a related cloud resource asset.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.
            include (str, optional:
                Comma-separated string; Include the provided fields as part of the serialized result.
                Allowed Values:
                    certDetails: populate all elements in data[*].details.
                    Note: If param is not specified, data[*].details will be empty.
            sort (str, optional):
                Comma-separated string; orders results by the given fields. If the field name is
                prefixed by a -, then the ordering will be descending for that field.
                Allowed values are `commonName`, `-commonName`, `dateAdded`, `-dateAdded`, `lastObserved`, `-lastObserved`.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the certificate results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Prints all certificate objects:
            >>> for res in client.assets.certificates.v2.list():
            ...     for cert in res:
            ...         print(cert)
        """
        return ExResultIterator(self._api, f"{V2_PREFIX}/assets/certificates", kwargs)

    def count(self, **kwargs: Any) -> int:
        """
        Returns the total count of certificates. This will return -1 if for some reason the count attribute
        is not returned in an otherwise valid response payload.

        Args:
            pageToken (str, optional):
                Returns results starting at this page Token.
            commonNameSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            formattedIssuerOrg (str, optional):
                Comma-separated string; Returns only results that were found on the given formatted issuer orgs.
                If not set, filter is ignored.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            property (str, optional):
                Comma-separated string; Returns only results whose certificate property falls in the provided list.
                NOTE: If omitted, API will return results for all properties the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            certificateAdvertisementStatus (str, optional):
                Comma-separated string; Returns only result whose asset's certificate advertisement statuses fall in the provided list.
                Valid values are `HAS_CERTIFICATE_ADVERTISEMENT` and `NO_CERTIFICATE_ADVERTISEMENT`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, and `NO_ACTIVE_ON_PREM_SERVICE`.
            issueStatus (str, optional):
                Comma-separated string; Returns only result whose asset's issue statuses fall in the provided list.
                Valid values are `New`, `Investigating`, `In Progress`, `No Risk`, `Acceptable Risk`, `Resolved`
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (boolean, optional):
                Filter by whether the asset has a related cloud resource asset.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.

        Returns:
            :int:
                The total count of certificates.

        Examples:
            >>> # Print total count of certificates containing `dev` substring.
            >>> print(client.assets.certificates.v2.count(commonNameSearch="dev"))
        """
        return (
            self._api.get(f"{V2_PREFIX}/assets/certificates/count", params=kwargs)
            .json()
            .get("count", -1)
        )

    def get(self, pemMd5Hash: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given Certificate. Arguments should be passed as keyword args using
        the names below.

        Args:
            pemMd5Hash (str):
                Fully qualified domain name.
            minRecentIpLastObservedDate (str, optional):
                Filter by last observed timestamp for recent IPs. Date formatted string (YYYY-MM-DD).

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the domain.

        Examples:
            >>> # Return Domain.
            >>> domain = client.assets.certificates.v2.get(<pemMd5Hash>)
        """
        return self._api.get(
            f"{V2_PREFIX}/assets/certificates/{pemMd5Hash}", params=kwargs
        ).json()

    def csv(self, file: str, **kwargs: Any):
        """
        Downloads filtered certificates as a csv file. Arguments should be passed as keyword args using
        the names below.

        Args:
            file (str):
                A relative path for saving the downloaded csv file.
            limit (int, optional):
                Returns at most this many results.
            commonNameSearch (str, optional):
                Search for given domain value via substring match.
            recentIp (str, optional):
                Filter by IP; Returns only assets with a recent IP matching the provided filter.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            formattedIssuerOrg (str, optional):
                Comma-separated string; Returns only results that were found on the given formatted issuer orgs.
                If not set, filter is ignored.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's Name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            property (str, optional):
                Comma-separated string; Returns only results whose certificate property falls in the provided list.
                NOTE: If omitted, API will return results for all properties the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            certificateAdvertisementStatus (str, optional):
                Comma-separated string; Returns only result whose asset's certificate advertisement statuses fall in the provided list.
                Valid values are `HAS_CERTIFICATE_ADVERTISEMENT` and `NO_CERTIFICATE_ADVERTISEMENT`.
            serviceStatus (str, optional):
                Comma-separated string; Returns only result whose asset's service statuses fall in the provided list.
                Valid values are `HAS_ACTIVE_SERVICE`, `NO_ACTIVE_SERVICE`, `HAS_ACTIVE_CLOUD_SERVICE`, `NO_ACTIVE_CLOUD_SERVICE`,
                `HAS_ACTIVE_ON_PREM_SERVICE`, and `NO_ACTIVE_ON_PREM_SERVICE`.
            issueStatus (str, optional):
                Comma-separated string; Returns only result whose asset's issue statuses fall in the provided list.
                Valid values are `New`, `Investigating`, `In Progress`, `No Risk`, `Acceptable Risk`, `Resolved`
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            hasRelatedCloudResources (boolean, optional):
                Filter by whether the asset has a related cloud resource asset.
            tagId (str, optional):
                Comma-separated string; Returns any assets with a tagId in the provided set.
            tagName (str, optional):
                Comma-separated string; Returns any assets with a tagName in the provided set.

        Returns:
            :obj:`boolean`:
                `True` if the download was successful, otherwise `False`.

        Examples:
            >>> # Download a csv named `api-certs.csv` for all certificates that contain `api` in their name:
            >>> cli.assets.certificates.v2.csv(file="api-certs.csv", commonNameSearch="api")
        """
        return self._api.csv(
            path=f"{V2_PREFIX}/assets/certificates/csv", file_=file, **kwargs
        )

    def bulk_tag(
        self,
        operation: str,
        asset_ids: List[str],
        tag_ids: List[str],
        return_raw: bool = False,
    ) -> Union[bool, Dict[str, Any]]:
        """
        Assigns or unassigns tags to assets.

        Args:
            operation (str):
                Operation type. Must be ASSIGN or UNASSIGN.
            asset_ids (list):
                A list of asset uuids to assign or unassign tags from.
            tag_ids (list):
                A list of tag uuids to assign or unassign to assets.
            return_raw (bool, optional):
                If False this will return a boolean response that reflects whether
                all of the operations were successful. If True the raw json response
                will be returned. Defaults to False.

        Returns:
            :bool:
                Returns a bool reflecting the operations success unless `return_raw`
                is true, in which case a dict is returned.

        Examples:
            >>> # Assign a tag to a certificate
            >>> cli.assets.certificates.bulk_tag("ASSIGN",
            ...                                  ["8e589910-c1af-3c32-ae88-9a4b2dbcfe76"],
            ...                                  ["f6164347-86d1-30cc-baf2-28bbb395403d"])
        """
        if operation not in ["ASSIGN", "UNASSIGN"]:
            raise UnexpectedValueError(
                f"The operation type '{operation}' is not valid.'"
            )
        payload: Dict[str, Any] = {"operations": []}
        for asset_id in asset_ids:
            payload["operations"].append(
                {"operationType": operation, "assetId": asset_id, "tagIds": tag_ids}
            )
        resp = self._api.post(
            f"{V2_PREFIX}/assets/certificates/tag-assignments/bulk", json=payload
        ).json()
        if not return_raw:
            return resp.get("meta", {}).get("failureCount") == 0
        return resp

    def bulk_poc(
        self,
        operation: str,
        asset_ids: List[str],
        contact_ids: List[str],
        return_raw: bool = False,
    ) -> Union[bool, Dict[str, Any]]:
        """
        Assigns or unassigns Point-of-Contacts to assets.

        Args:
            operation (str):
                Operation type. Must be ASSIGN or UNASSIGN.
            asset_ids (list):
                A list of asset uuids to assign or unassign pocs from.
            contact_ids (list):
                A list of poc uuids to assign or unassign to assets.
            return_raw (bool, optional):
                If False this will return a boolean response that reflects whether
                all of the operations were successful. If True the raw json response
                will be returned. Defaults to False.

        Returns:
            :bool:
                Returns a bool reflecting the operations success unless `return_raw`
                is true, in which case a dict is returned.

        Examples:
            >>> # Assign a poc to a certificate
            >>> cli.assets.certificates.bulk_poc("ASSIGN",
            ...                                  ["8e589910-c1af-3c32-ae88-9a4b2dbcfe76"],
            ...                                  ["f6164347-86d1-30cc-baf2-28bbb395403d"])
        """
        if operation not in ["ASSIGN", "UNASSIGN"]:
            raise UnexpectedValueError(
                f"The operation type '{operation}' is not valid.'"
            )
        payload: Dict[str, Any] = {"operations": []}
        for asset_id in asset_ids:
            payload["operations"].append(
                {
                    "operationType": operation,
                    "assetId": asset_id,
                    "contactIds": contact_ids,
                }
            )
        resp = self._api.post(
            f"{V2_PREFIX}/assets/certificates/contact-assignments/bulk", json=payload
        ).json()
        if not return_raw:
            return resp.get("meta", {}).get("failureCount") == 0
        return resp

    def annotation_update(
        self,
        certificate_id: str,
        contacts: List[str] = [],
        tags: List[str] = [],
        note: str = "",
    ) -> Dict[str, Any]:
        """
        Updates the annotations for a single certificate.

        Note: This will overwrite the existing annotations for an asset.
              If any arguments are omitted, that annotation type will be
              cleared for the asset.

        Args:
            certificate_id (str):
                The uuid of the certificate.
            contacts (list, optional):
                A list of poc emails to apply to the certificate.
            tags (list, optional):
                A list of tag names to apply to the certificate.
            note (str, optional):
                An optional note that can be added to the certificate.

        Returns:
            :obj:`dict`:
                A dictionary containing the current annotations for the certificate.

        Examples:
            >>> # Update annotations for a certificate
            >>> cli.assets.certificates.annotation_update(
            ...     certificate_id="e5bdc732-522a-3864-8ff3-307d35f0f0a0",
            ...     contacts=["QA@testing.com"],
            ...     tags=["sdk_test"],
            ...     note="SDK Note")
        """
        payload: Dict[str, Any] = {"tags": [], "note": note or "", "contacts": []}
        for contact in contacts:
            payload["contacts"].append({"email": contact})
        for tag in tags:
            payload["tags"].append({"name": tag})
        return self._api.put(
            f"{V2_PREFIX}/assets/certificates/{certificate_id}/annotations",
            json=payload,
        ).json()



from enum import Enum


class HTTPVerb(Enum):
    """Enums for HTTP verbs"""

    HTTP_GET = "GET"
    """HTTP GET Verb"""

    HTTP_PATCH = "PATCH"
    """HTTP PATCH Verb"""

    HTTP_POST = "POST"
    """HTTP POST Verb"""

    HTTP_PUT = "PUT"
    """HTTP PUT Verb"""

    HTTP_DELETE = "DELETE"
    """HTTP DELETE Verb"""


XPANSE_BEARER_TOKEN = "XPANSE_BEARER_TOKEN"
"""Env Variable for Bearer Token"""

XPANSE_JWT_TOKEN = "XPANSE_JWT_TOKEN"
"""Env Variable for JWT"""

ID_TOKEN_URL = "api/v1/idtoken"
"""Token Refresh URL"""

V1_PREFIX = "api/v1"
"""V1 URL Prefix"""

V2_PREFIX = "api/v2"
"""V2 URL Prefix"""

V3_PREFIX = "api/v3"
"""V3 URL Prefix"""



from typing import Any, Dict
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint


class TargetedIPsEndpoint(ExEndpoint):
    """
    Part of the Targeted IPs v1 API for accessing scan origination IPs.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self) -> Dict[str, Any]:
        """
        Returns a list of IPs used in Xpanse Scanning.

        Returns:
            :obj:`dict`:
                An dictionary containing a `lastUpdated` which related to when the list was last updated
                and a `prefixes` which is a list of dicts containing the scanner IP prefix.

        Examples:
            >>> # Prints all scanner prefixes:
            >>> for prefix in client.targeted_ips.targeted_ips.list().get("prefixes"):
            ...     print(prefix.get("prefix"))
        """
        return self._api.get(f"{V1_PREFIX}/targetedIps", params={}).json()



from typing import Any, Dict, List, Tuple

from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.error import UnexpectedValueError
from xpanse.iterator import ExResultIterator

VALID_UPDATE_TYPES = {
    "Assignee",
    "Comment",
    "Priority",
    "ProgressStatus",
}


class IssuesEndpoint(ExEndpoint):
    """
    Part of the Issues V1 API.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        This endpoint will return a paginated list of issues.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            issueTypeId (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            issueTypeName (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            inetSearch (str, optional):
                Search for results in a given IP/CIDR block using a single IP (d.d.d.d), a dashed IP range (d.d.d.d-d.d.d.d),
                a CIDR block (d.d.d.d/m), a partial CIDR (d.d.), or a wildcard (d.d.*.d).
                Returns results whose identifier includes an IP matching the query.
            domainSearch (str, optional):
                Search for a a given domain value via substring match.
                Returns results whose identifier includes a domain matching the query.
            portNumber (str, optional):
                Comma-separated string; Returns only results whose identifier includes one of the given port numbers.
            progressStatus (str, optional):
                Comma-separated string; Returns only results whose progress status matches one of the given values.
                Allowed values are `New`, `Investigating`, `InProgress`, `AcceptableRisk`, `Resolved`.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            tagName (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
            tagId (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag id should be used here rather than the tag name.
            createdAfter (str, optional):
                Returns only results created after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            createdBefore (str, optional):
                Returns only results created before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedAfter (str, optional):
                Returns only results modified after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedBefore (str, optional):
                Returns only results modified before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            countryCode (str, optional):
                Comma-separated string of ISO-3166 two character country codes; Returns any assets with an IP located in a country in the provided set.
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            sort (str, optional):
                Sort by specified properties.
                Allowed values are `created`, `-created`, `modified`, `-modified`, `assigneeUsername`, `-assigneeUsername`,
                `priority`, `-priority`, `progressStatus`, `-progressStatus`, `activityStatus`, `-activityStatus`, `headline`, and `-headline`.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the issues results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all issues dumped to a list:
            >>> bus =  client.issues.issues.list().dump()
        """
        return ExResultIterator(self._api, f"{V1_PREFIX}/issues/issues", kwargs)

    def count(self, **kwargs: Any) -> Dict[str, Any]:
        """
        Get a count of issues. Returns the total count of issues matching the provided filters, up to 10K.

        Args:
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            issueTypeId (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            issueTypeName (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            inetSearch (str, optional):
                Search for results in a given IP/CIDR block using a single IP (d.d.d.d), a dashed IP range (d.d.d.d-d.d.d.d),
                a CIDR block (d.d.d.d/m), a partial CIDR (d.d.), or a wildcard (d.d.*.d).
                Returns results whose identifier includes an IP matching the query.
            domainSearch (str, optional):
                Search for a a given domain value via substring match.
                Returns results whose identifier includes a domain matching the query.
            portNumber (str, optional):
                Comma-separated string; Returns only results whose identifier includes one of the given port numbers.
            progressStatus (str, optional):
                Comma-separated string; Returns only results whose progress status matches one of the given values.
                Allowed values are `New`, `Investigating`, `InProgress`, `AcceptableRisk`, `Resolved`.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            tag (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
            createdAfter (str, optional):
                Returns only results created after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            createdBefore (str, optional):
                Returns only results created before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedAfter (str, optional):
                Returns only results modified after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedBefore (str, optional):
                Returns only results modified before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.

        Returns:
            :obj:`dict`:
                A dictionary containing count and overflow details.

        Examples:
            >>> # Return total issue count for assets with the `validated` tag.
            >>> bus =  client.issues.issues.v1.count(tag="validated")
        """
        return self._api.get(f"{V1_PREFIX}/issues/issues/count", params=kwargs).json()

    def counts(self, include="issueTypeId", **kwargs):
        """
        Get bulk counts of issues. Returns the counts of issues matching the provided filters for each value of the fields specified by the include parameter,
        up to 100 per value. At this time, the only supported value for the include parameter is issueTypeId.
        Filters for the field specified by the include parameter are ignored when computing counts for that field.

        Args:
            include (str):
                Comma-separated string; Include counts for all possible values of the provided fields.
                Allowed values are `issueTypeId`.
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            issueTypeId (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            issueTypeName (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            inetSearch (str, optional):
                Search for results in a given IP/CIDR block using a single IP (d.d.d.d), a dashed IP range (d.d.d.d-d.d.d.d),
                a CIDR block (d.d.d.d/m), a partial CIDR (d.d.), or a wildcard (d.d.*.d).
                Returns results whose identifier includes an IP matching the query.
            domainSearch (str, optional):
                Search for a a given domain value via substring match.
                Returns results whose identifier includes a domain matching the query.
            portNumber (str, optional):
                Comma-separated string; Returns only results whose identifier includes one of the given port numbers.
            progressStatus (str, optional):
                Comma-separated string; Returns only results whose progress status matches one of the given values.
                Allowed values are `New`, `Investigating`, `InProgress`, `AcceptableRisk`, `Resolved`.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            tag (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
            createdAfter (str, optional):
                Returns only results created after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            createdBefore (str, optional):
                Returns only results created before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedAfter (str, optional):
                Returns only results modified after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedBefore (str, optional):
                Returns only results modified before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.


        Returns:
            :obj:`dict`:
                A dictionary containing field count and overflow details.

        Examples:
            >>> # Return total issue counts
            >>> bus =  client.issues.issues.v1.counts()
        """
        kwargs["include"] = include
        return self._api.get(f"{V1_PREFIX}/issues/issues/counts", params=kwargs).json()

    def get(self, id, **kwargs):
        """
        Returns the details for a given Issue. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested issue.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the issue.

        Examples:
            >>> # Return Issue.
            >>> issue = client.issues.issues.get(<id>)
        """
        return self._api.get(f"{V1_PREFIX}/issues/issues/{id}", params=kwargs).json()

    def get_updates(self, id: str, **kwargs: Any) -> ExResultIterator:
        """
        Returns the issue updates for a specified issue. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested issue.
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the issue's updates.

        Examples:
            >>> # Return updates for issue and dump to list.
            >>> issue_updates = client.issues.issues.get_updates(<id>).dump()
        """
        return ExResultIterator(
            self._api, f"{V1_PREFIX}/issues/issues/{id}/updates", kwargs
        )

    def get_update(self, id: str, update_id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the update details for a specified issue and update. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested issue.
            update_id (str):
                ID of the requested issue update.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the issue update.

        Examples:
            >>> # Return issue update details
            >>> issue_update = client.issues.issues.get_update(id=<id>, update_id=<update_id>)
        """
        return self._api.get(
            f"{V1_PREFIX}/issues/issues/{id}/updates/{update_id}", params=kwargs
        ).json()

    def update(
        self, id: str, value: str, updateType: str, **kwargs: Any
    ) -> Dict[str, Any]:
        """
        Make an update to an issue. Requires a value and an updateType.
        Valid updateType values include: `Assignee`, `Comment`, `Priority`, or `ProgressStatus`.
        Valid values will vary based on the updateType, with some being limited and others being open text fields.
        Assignee - `value` must be an existing username.
        Comment - `value` can be any plaintext.
        Priority - `value` can be `Low`, `Medium`, `High`, or `Critical`
        ProgressStatus - `value` can be `New`, `Investigating`, `InProgress`, `AcceptableRisk`, or `Resolved`

        Args:
            id (str):
                ID of the requested issue.
            value (str):
                The value to be updated.
            updateType (str):
                The update type to be performed.
                Allowed types are `Assignee`, `Comment`, `Priority`, or `ProgressStatus`.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the issue's updates.

        Examples:
            >>> # Return updates for issue and dump to list.
            >>> updates = client.issues.issues.update(id=<id>, value=<username>, updateType="Assignee")
        """
        if id is None or value is None or updateType is None:
            raise UnexpectedValueError("A required update value was missing")
        if updateType not in VALID_UPDATE_TYPES:
            raise UnexpectedValueError(
                f"updateType '{updateType}' was not found in list of allowed types: {VALID_UPDATE_TYPES}"
            )
        payload = {"value": value, "updateType": updateType}
        return self._api.post(
            f"{V1_PREFIX}/issues/issues/{id}/updates", json=payload
        ).json()

    def bulk_update(
        self, updates: List[Tuple[str, str, str]], **kwargs: Any
    ) -> Dict[str, Any]:
        """
        Makes updates to multiple issues. Requires an id, value, and updateType for each issue.
        Valid updateType values include: `Assignee`, `Comment`, `Priority`, or `ProgressStatus`.
        Valid values will vary based on the updateType, with some being limited and others being open text fields.
        Assignee - `value` must be an existing username.
        Comment - `value` can be any plaintext.
        Priority - `value` can be `Low`, `Medium`, `High`, or `Critical`
        ProgressStatus - `value` can be `New`, `Investigating`, `InProgress`, `AcceptableRisk`, or `Resolved`

        Args:
            updates (list):
                A list of tuples containing (`id`, `value`, `updateType`), where id is the issue ID, value is the assigned value
                and updateType is one of the following values `Assignee`, `Comment`, `Priority`, `ProgressStatus`, or `ActivityStatus`.

        Returns:
            :obj:`dict`:
                A dictionary containing details about the bulk update's execution.

        Examples:
            >>> # Return updates for issue and dump to list.
            >>> response = client.issues.issues.bulk_update([('7320d57a-a39f-3bae-beae-9e56b1cf95cc', 'InProgress', 'ProgressStatus')])
        """
        payload = []
        for items in updates:
            id_, value, updateType = items
            if id_ is None or value is None or updateType is None:
                raise UnexpectedValueError(f"A required update value was missing")
            if updateType not in VALID_UPDATE_TYPES:
                raise UnexpectedValueError(
                    f"updateType '{updateType}' was not found in list of allowed types: {VALID_UPDATE_TYPES}"
                )
            payload.append(
                {
                    "issueId": id_,
                    "updateRequest": {"value": value, "updateType": updateType},
                }
            )
        return self._api.post(
            f"{V1_PREFIX}/issues/issues/bulk", json={"operations": payload}
        ).json()

    def csv(self, file: str, **kwargs: Any) -> bool:
        """
        This endpoint will export a filtered list of issues to csv.

        Args:
            file (str):
                The name of the returned CSV file.
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            businessUnitName (str, optional):
                Comma-separated string; Returns only results whose Business Unit's name falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            issueTypeId (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            issueTypeName (str, optional):
                Comma-separated string; Returns only results whose issue type name matches one of the given types.
            inetSearch (str, optional):
                Search for results in a given IP/CIDR block using a single IP (d.d.d.d), a dashed IP range (d.d.d.d-d.d.d.d),
                a CIDR block (d.d.d.d/m), a partial CIDR (d.d.), or a wildcard (d.d.*.d).
                Returns results whose identifier includes an IP matching the query.
            domainSearch (str, optional):
                Search for a a given domain value via substring match.
                Returns results whose identifier includes a domain matching the query.
            portNumber (str, optional):
                Comma-separated string; Returns only results whose identifier includes one of the given port numbers.
            progressStatus (str, optional):
                Comma-separated string; Returns only results whose progress status matches one of the given values.
                Allowed values are `New`, `Investigating`, `InProgress`, `AcceptableRisk`, `Resolved`.
            activityStatus (str, optional):
                Comma-separated string; Returns only results whose activity status matches one of the given values.
                Allowed values are `Active` and `Inactive`.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            tag (str, optional):
                Comma-separated string; Returns only results that are associated with the provided tags.
                The tag name should be used here rather than the tag id.
            createdAfter (str, optional):
                Returns only results created after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            createdBefore (str, optional):
                Returns only results created before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedAfter (str, optional):
                Returns only results modified after the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            modifiedBefore (str, optional):
                Returns only results modified before the provided timestamp (YYYY-MM-DDTHH:MM:SSZ).
            cloudManagementStatus (str, optional):
                Comma-separated string; Returns only results whose cloud management status matches one of the given values.
                Allowed values are `NotApplicable`, `ManagedCloud`, `UnmanagedCloud`.
            sort (str, optional):
                Sort by specified properties.
                Allowed values are `created`, `-created`, `modified`, `-modified`, `assigneeUsername`, `-assigneeUsername`,
                `priority`, `-priority`, `progressStatus`, `-progressStatus`, `activityStatus`, `-activityStatus`, `headline`, and `-headline`.

        Returns:
            :obj:`boolean`:
                `True` if the download was successful, otherwise `False`.

        Examples:
            >>> # Download a csv named `insecure_tls.csv` for all 'InsecureTLS' issues.
            >>> client.issues.issues.csv(file="insecure_tls.csv", issueTypeId="InsecureTLS")
        """
        return self._api.csv(
            path=f"{V1_PREFIX}/issues/issues/csv", file_=file, **kwargs
        )



from xpanse.base_api import ExApi
from xpanse.api.issues.v1.assignees import AssigneesEndpoint
from xpanse.api.issues.v1.business_units import BusinessUnitsEndpoint
from xpanse.api.issues.v1.issues import IssuesEndpoint
from xpanse.api.issues.v1.issue_types import IssueTypesEndpoint
from xpanse.api.issues.v1.policies import PoliciesEndpoint
from xpanse.api.issues.v1.providers import ProvidersEndpoint
from xpanse.api.issues.v1.updates import UpdatesEndpoint


class IssuesAssigneesAPI(ExApi, AssigneesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return AssigneesEndpoint(self._api)


class IssuesBusinessUnitsAPI(ExApi, BusinessUnitsEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return BusinessUnitsEndpoint(self._api)


class IssuesIssuesAPI(ExApi, IssuesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return IssuesEndpoint(self._api)


class IssuesIssueTypesAPI(ExApi, IssueTypesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return IssueTypesEndpoint(self._api)


class IssuesPoliciesAPI(ExApi, PoliciesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return PoliciesEndpoint(self._api)


class IssuesProvidersAPI(ExApi, ProvidersEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return ProvidersEndpoint(self._api)


class IssuesUpdatesAPI(ExApi, UpdatesEndpoint):
    def __init__(self, session):
        super().__init__(session)

    @property
    def current_version(self):
        return "v1"

    @property
    def v1(self):
        return UpdatesEndpoint(self._api)


class IssuesApi(ExApi):
    @property
    def assignees(self):
        return IssuesAssigneesAPI(self._api)

    @property
    def business_units(self):
        return IssuesBusinessUnitsAPI(self._api)

    @property
    def issues(self):
        return IssuesIssuesAPI(self._api)

    @property
    def issue_types(self):
        return IssuesIssueTypesAPI(self._api)

    @property
    def policies(self):
        return IssuesPoliciesAPI(self._api)

    @property
    def providers(self):
        return IssuesProvidersAPI(self._api)

    @property
    def updates(self):
        return IssuesUpdatesAPI(self._api)



from typing import Any, Dict, Optional
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.error import UnexpectedValueError
from xpanse.iterator import ExResultIterator


class PoliciesEndpoint(ExEndpoint):
    """
    Part of the Issues V1 API.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    DEFAULT_ASSIGNEE = "Unassigned"
    DEFAULT_PRIORITY = "Low"
    DEFAULT_ENABLED_STATUS = "Off"
    ALLOWED_PRIORITIES = {"Low", "Medium", "High", "Critical"}
    ALLOWED_ENABLED_STATUSES = {"On", "Off"}

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        This endpoint will return a paginated list of issue policies.

        Args:
            limit (int, optional):
                Returns at most this many results in a single api call.
                Default is 100, max is 10000.
            pageToken (str, optional):
                Page token for pagination.
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            enabledStatus (str, optional):
                Comma-separated string; Returns only results with enabled status matching the provided value.
                Allowed values are `Off`, `On`.
            category (str, optional):
                Comma-separated string; Returns only results whose category matches one of the given categories.
            sort (str, optional):
                The field to use to sort the returned records. Prefix with '-' to reverse the normal sort order.
                Allowed values are `issueTypeName`, `-issueTypeName`,`issueTypeId`, `-issueTypeId`, `category`, `-category`,
                `created`, `-created`, `modified`, `-modified`, `modifiedBy`, `-modifiedBy`, `priority`, `-priority`,
                `assigneeUsername`, `-assigneeUsername`, `enabledStatus`, `-enabledStatus`.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the issue policy results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all issue policies dumped to a list:
            >>> bus =  client.issues.policies.list().dump()
        """
        return ExResultIterator(self._api, f"{V1_PREFIX}/issues/policies", kwargs)

    def count(self, **kwargs: Any) -> Dict[str, Any]:
        """
        This endpoint will return a count of all issues that match the specified filters.

        Args:
            contentSearch (str, optional):
                Returns only results whose contents match the given query.
            priority (str, optional):
                Comma-separated string; Returns only results whose priority matches one of the given values.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            assigneeUsername (str, optional):
                Comma-separated string; Returns only results whose assignee's username matches one of the given usernames.
                Use "Unassigned" to fetch issues that are not assigned to any user.
            enabledStatus (str, optional):
                Comma-separated string; Returns only results with enabled status matching the provided value.
                Allowed values are `Off`, `On`.
            category (str, optional):
                Comma-separated string; Returns only results whose category matches one of the given categories.

        Returns:
            :obj:`dict`:
                A dictionary containing count and overflow details.

        Examples:
            >>> # Return total policy count for enabled policies.
            >>> bus =  client.issues.policies.count(enabledStatus=)
        """
        return self._api.get(f"{V1_PREFIX}/issues/policies/count", params=kwargs).json()

    def get(self, id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        Returns the details for a given Policy. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested policy. This will be its `issueTypeId`.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the policy.

        Examples:
            >>> # Return Policy.
            >>> issue = client.issues.policies.get(<id>)
        """
        return self._api.get(f"{V1_PREFIX}/issues/policies/{id}", params=kwargs).json()

    def update(
        self,
        id: str,
        assignee: Optional[str] = None,
        priority: Optional[str] = None,
        enabled: Optional[str] = None,
    ) -> Dict[str, Any]:
        """
        Updates a policy.
        NOTE: This is an overwrite operation. Please specify all desired values when updating a policy, even if the value is not changing.

        Args:
            id (str):
                ID of the requested policy. This will be its `issueTypeId`.
            assignee (str, optional):
                The new default assigneeUsername for the policy. If unused it will default to `Unassigned`.
            priority (str, optional):
                The new default priority for the policy. If unused it will default to `Low`.
                Allowed values are `Critical`, `High`, `Medium`, and `Low`.
            enabled (str, optional):
                The new default enabledStatus. If unused it will default to `Off`.
                Allowed values are `Off`, `On`.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the updated details about the policy.

        Examples:
            >>> # Update the Microsoft DNS Server Policy to be enabled and set to Medium.
            >>> issue = client.issues.policies.update(id="MicrosoftDnsServer", priority="Medium", enabled="On")
        """
        if priority is not None and priority not in self.ALLOWED_PRIORITIES:
            raise UnexpectedValueError(
                f"priority '{priority}' was not found in list of allowed types: {self.ALLOWED_PRIORITIES}"
            )
        if enabled is not None and enabled not in self.ALLOWED_ENABLED_STATUSES:
            raise UnexpectedValueError(
                f"enabled '{enabled}' was not found in list of allowed types: {self.ALLOWED_ENABLED_STATUSES}"
            )
        payload = {
            "assigneeUsername": assignee or self.DEFAULT_ASSIGNEE,
            "priority": priority or self.DEFAULT_PRIORITY,
            "enabledStatus": enabled or self.DEFAULT_ENABLED_STATUS,
        }
        return self._api.put(f"{V1_PREFIX}/issues/policies/{id}", json=payload).json()



from typing import Any, Dict
from xpanse.const import V1_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class ConnectorsAccountsEndpoint(ExEndpoint):
    """
    Part of the Connectors V1 API.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
           This endpoint will return a paginated list of connector accounts.

           Args:
               limit (int, optional):
                   Returns at most this many results in a single api call.
                   Default is 100, max is 10000.
               pageToken (str, optional):
                   Page token for pagination.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the connector accounts results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all connector sercices dumped to a list:
            >>> bus =  client.connectors.accounts.list().dump()
        """

        return ExResultIterator(self._api, f"{V1_PREFIX}/connectors/accounts", kwargs)

    def get(self, id: str, **kwargs: Any) -> Dict[str, Any]:
        """
        This endpoint will return details for a given connector account. Arguments should be passed as keyword args using
        the names below.

        Args:
            id (str):
                ID of the requested connector account.

        Returns:
            :obj:`dict`:
                A dictionary containing all of the details about the connector account.

        Examples:
            >>> # Return Issue.
            >>> issue = client.connector.accounts.get(<id>)
        """
        return self._api.get(
            f"{V1_PREFIX}/connectors/accounts/{id}", params=kwargs
        ).json()



import logging
from typing import Any, Dict, List, Optional

from xpanse.error import UnexpectedResponseError


class ExResultIterator:
    """
    Iterator for paging though results.
    """

    # Total number of results - may not be available
    _total: int = 0

    # Pages seen so far
    _pages: int = 0

    # Location of next page
    _next_url: Optional[str] = None

    def __init__(self, api: Any, path: str, args: Dict[str, Any]):
        self._api = api
        self._path = path
        self._params = self._clean_params(args)
        self._log = logging.getLogger(
            "{}.{}".format(self.__module__, self.__class__.__name__)
        )

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    @property
    def total(self) -> int:
        return self._total

    def next(self) -> Dict[str, Any]:
        """
        Fetches the result from pagination.next, if a value exists.
        """
        if self._next_url is None and self._pages >= 1:
            raise StopIteration("Pagination exhausted")

        return self._get_data()

    def dump(self) -> List[Any]:
        """
        Iterates until completion and returns a list of results.
        """
        return [j for i in self for j in i]

    def _get_data(self) -> Dict[str, Any]:
        """
        Returns the next page of data
        """
        try:
            if self._pages >= 1:
                resp = self._api.direct_get(self._next_url)
            else:
                resp = self._api.get(self._path, params=self._params)
            resp_as_json = resp.json()  # type: ignore

            self._pages += 1
            self._next_url = resp_as_json.get("pagination", {}).get("next", None)
            self._total = (
                resp_as_json.get("meta").get("totalCount", 0)
                if resp_as_json.get("meta") is not None
                else 0
            )
            return resp_as_json["data"]
        except KeyError as err:
            raise UnexpectedResponseError(
                "ExResultIterator received unexpected response"
            ) from err

    def _clean_params(self, params: Dict[str, Any]) -> Dict[str, Any]:
        """
        Some param names utilize python required keywords.
        """
        if "type_" in params:
            params["type"] = params.pop("type_")
        return params



from typing import Any

from xpanse.const import V2_PREFIX
from xpanse.endpoint import ExEndpoint
from xpanse.iterator import ExResultIterator


class IpsEndpoint(ExEndpoint):
    """
    Part of the Assets v2 API for handling Recent IPs.
    See: https://api.expander.expanse.co/api/v1/docs/
    """

    def list(self, **kwargs: Any) -> ExResultIterator:
        """
        Returns the list of Recent IPs. Arguments should be passed as keyword args using
        the names below.

        Args:
            assetKey (str, optional):
                Filter by asset key.
            assetType (str, optional):
                Filter by asset type.
            inetSearch (str, optional):
                Filter by IP, IP range, IP CIDR or IP wildcard.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.
            sort (str, optional):
                Comma-separated string; orders results by the given fields. If the field name is
                prefixed by a -, then the ordering will be descending for that field.
            limit (int, optional):
                Returns at most this many results in a single api call.
            pageToken (str, optional):
                Returns results starting at this Page Token.

        Returns:
            :obj:`ExResultIterator`:
                An iterator containing all of the ip results. Results can be iterated
                or called by page using `<iterator>.next()`.

        Examples:
            >>> # Return all ips and print each resource:
            >>> for ips in client.assets.ips.v2.list():
            ...     for resource in ips:
            ...         print(resource)
        """
        return ExResultIterator(self._api, f"{V2_PREFIX}/assets/ips", kwargs)

    def csv(self, file: str, **kwargs: Any) -> bool:
        """
        Download a CSV file of asset IPs. This endpoint contains the IP addresses that your organization’s domains
        resolve to and certificates are advertised on. For on-premise IP ranges attributed to your organization,
        please use the `IpRangeEndpoint.list()` method (defaults 30 days).

        Downloads filtered ips as a csv file. Arguments should be passed as keyword args using
        the names below.

        Args:
            file (str):
                A relative path for saving the downloaded csv file.
            assetKey (str, optional):
                Filter by asset key.
            assetType (str, optional):
                Filter by asset type.
            providerId (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            providerName (str, optional):
                Comma-separated string; Returns only results that were found on the given providers.
                If not set, results will include anything regardless of provider status.
            businessUnitId (str, optional):
                Comma-separated string; Returns only results whose Business Unit's ID falls in the provided list.
                NOTE: If omitted, API will return results for all Business Units the user has permissions to view.
            minLastObservedDate (str, optional):
                Filter by last observed timestamp. Date formatted string (YYYY-MM-DD).
            hostingEnvironment (str, optional):
                Filter by Hosting Environment. Allowed values are `ON_PREM`, `CLOUD`, `NONE`, `RESERVED_IPS`.

        Returns:
            :obj:`boolean`:
                `True` if the download was successful, otherwise `False`.

        Examples:
            >>> # Download a csv named `cloud-ips.csv` for all ips hosted in the cloud.
            >>> client.assets.ips.v2.csv(file="cloud-ips.csv", hostingEnvironment="CLOUD")
        """
        return self._api.csv(path=f"{V2_PREFIX}/assets/ips/csv", file_=file, **kwargs)
